<?php

namespace Drupal\Proc\Form;

use Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Component\Utility\Html;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Session\AccountProxy;
use Drupal\proc\Entity\Proc;
use Drupal\proc\ProcKeyManagerInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Base class for Proc Cryptographic Operation forms.
 */
abstract class ProcOpFormBase extends FormBase {

  /**
   * The logger channel.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected ?LoggerInterface $logger;

  /**
   * The ProcKeyManager service.
   *
   * @var \Drupal\proc\ProcKeyManagerInterface
   */
  protected ProcKeyManagerInterface $procKeyManager;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected ?EntityTypeManagerInterface $entityTypeManager;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxy
   */
  protected $currentUser;

  /**
   * ProcEncryptForm form.
   *
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger channel.
   * @param \Drupal\proc\ProcKeyManagerInterface $proc_key_manager
   *   The ProcKeyManager service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\Core\Session\AccountProxy $current_user
   *   The current user.
   */
  public function __construct(
    LoggerInterface $logger,
    ProcKeyManagerInterface $proc_key_manager,
    EntityTypeManagerInterface $entityTypeManager,
    AccountProxy $current_user,
  ) {
    $this->logger = $logger;
    $this->procKeyManager = $proc_key_manager;
    $this->entityTypeManager = $entityTypeManager;
    $this->currentUser = $current_user;
  }

  /**
   * {@inheritdoc}
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The Drupal service container.
   *
   * @return static
   */
  public static function create(ContainerInterface $container): static {
    return new static(
      $container->get('logger.factory')->get('proc'),
      $container->get('proc.key_manager'),
      $container->get('entity_type.manager'),
      $container->get('current_user'),
    );
  }

  /**
   * Create or update the Proc entity.
   *
   * @param bool|int|null $default_proc
   *   The default proc ID.
   *
   * @return \Drupal\Core\Entity\EntityInterface
   *   The Proc entity.
   *
   * @SuppressWarnings(PHPMD.StaticAccess)
   */
  public function createOrUpdateProcEntity(bool|int|null $default_proc): EntityInterface {
    $proc = Proc::create();

    if ($default_proc && is_numeric($default_proc)) {
      $proc_entity = $this->entityTypeManager->getStorage('proc')->load($default_proc);
      if ($proc_entity && $proc_entity->getOwnerId() == $this->currentUser->id()) {
        $proc = $proc_entity;
      }
    }
    return $proc;
  }

  /**
   * Get the cipher.
   *
   * @param array $cids
   *   The cipher IDs.
   * @param bool|null $metadata_only
   *   Whether to get only the metadata.
   *
   * @return array
   *   Array containing ciphers, encrypted privkey and recipient's keyring type.
   *
   * @SuppressWarnings(PHPMD.StaticAccess)
   */
  public function getCipher(array $cids, bool $metadata_only = NULL): array {
    $user_id = parent::currentUser()->id();

    try {
      $keyring = $this->procKeyManager->getKeys($user_id, 'user_id');
    }
    catch (InvalidPluginDefinitionException | PluginNotFoundException $e) {
      $this->logger('proc')->error($e->getMessage());
      return [];
    }

    $privkey = $keyring['encrypted_private_key'];
    $pubkey = $keyring['public_key'];

    $cid_cipher = [];

    foreach ($cids as $cid) {
      $cid_cipher[$cid] = self::getArmoured((int) $cid, NULL, NULL, $metadata_only);
    }
    return [
      'privkey' => $privkey,
      'keyring_cid' => $keyring['keyring_cid'],
      'ciphers' => $cid_cipher,
      'keyring_type' => $keyring['keyring_type'],
      'pubkey' => $pubkey,
    ];

  }

  /**
   * Get given ciphertext decryptable by the current user.
   *
   * @param string $cid
   *   Ciphertext ID (proc entity).
   * @param mixed|null $form
   *   Optional form parameter.
   * @param mixed|null $form_state
   *   Optional form state parameter.
   * @param bool|null $metadata_only
   *   Whether to return only metadata.
   *
   * @return array
   *   Array containing ciphertext data.
   */
  public function getArmoured(string $cid, mixed $form = NULL, mixed $form_state = NULL, bool $metadata_only = NULL): array {
    try {
      $entity = $this->entityTypeManager->getStorage('proc')->load($cid);
    }
    catch (InvalidPluginDefinitionException | PluginNotFoundException $e) {
      // Log error:
      $this->logger('proc')->error($e->getMessage());
      return [];
    }
    if (!$entity) {
      return [];
    }
    $recipient_id = $this->getRecipientIds($entity);
    $user_id = parent::currentUser()->id();

    $cipher_text_data = [];
    if (!$metadata_only && isset($entity->get('armored')->getValue()[0]['cipher_fid'])) {
      $cipher_text_data['cipher_text'] = $entity->get('armored')->getValue()[0]['cipher_fid'];
    }
    if (in_array($user_id, $recipient_id) || $metadata_only) {
      $cipher_text_data = array_merge($cipher_text_data, self::getMetadata($entity, $cid, $recipient_id));
    }
    return $cipher_text_data;
  }

  /**
   * Get recipient IDs from the entity.
   *
   * @param object $entity
   *   The proc entity.
   *
   * @return array
   *   Array of recipient IDs.
   */
  public function getRecipientIds(object $entity): array {
    $recipient_id = [];
    foreach ($entity->get('field_recipients_set')->getValue() as $recipient) {
      $recipient_id[] = $recipient['target_id'];
    }
    return $recipient_id;
  }

  /**
   * Get metadata from the entity.
   *
   * @param object $entity
   *   The proc entity.
   * @param string $cid
   *   Ciphertext ID (proc entity).
   * @param array $recipient_id
   *   Array of recipient IDs.
   *
   * @return array
   *   Array containing metadata.
   *
   * @SuppressWarnings(PHPMD.StaticAccess)
   */
  public function getMetadata(object $entity, string $cid, array $recipient_id): array {
    $meta = $entity->get('meta')->getValue()[0];
    return [
      'source_file_name' => isset($meta['source_file_name']) ? Html::escape($meta['source_file_name']) : '',
      'source_file_size' => isset($meta['source_file_size']) ? Html::escape($meta['source_file_size']) : '',
      'cipher_cid' => $cid,
      'proc_owner_uid' => $entity->get('user_id')->getValue()[0]['target_id'],
      'proc_recipients' => $recipient_id,
      'changed' => $entity->get('changed')->getValue()[0]['value'],
    ];
  }

}
